const Workspace = require('smartfacecloud-emulator-index');
const baseImageServePath = "/ui-editor/img";
const path = require("path");
const fs = require('fs');
const lwip = require('lwip');
const LogToConsole = require('../../common/LogToConsole');
const CONSTANTS = require("../../constants");

module.exports = function(app, options) {
    options = options || {};
    var log = new LogToConsole(options.logToConsole, '[IMAGE]').log;
    app.get(baseImageServePath + "/:imageName", function(req, res, next) {
        var wsPath = CONSTANTS.WORKSPACE_PATH;
        var device = {
                os: req.query.os,
                screen: {
                    dp: req.query.density
                },
                brandModel: req.query.brandModel
            },
            densityRatio = req.query.densityRatio || 1;
        try {
            if (req.query.resourceFolderOrder) {
                var resourceFolderOrder = req.query.resourceFolderOrder.split(",");
                for (var i = 0; i < resourceFolderOrder.length; i++) {
                    resourceFolderOrder[i] = "\"" + resourceFolderOrder[i] + "\"";
                }
                var functionBody = "return [" + resourceFolderOrder + "];";
                var fn = new Function(functionBody);
                device.resourceFolderOrder = fn();
            }
        }
        catch (ex) {
            debugger;
        }

        var zoomLevel = Number(req.query.zoomLevel) || 1;
        var ws = new Workspace({
            path: wsPath,
            projectID: process.env.C9_HOSTNAME
        });
        var imageName = req.params.imageName;

        if (imageName !== "*") {
            ws.getImage(device, req.params.imageName,
                function indexResult(err, index) {
                    if (err) {
                        res.sendStatus(500);
                        return log("**ERROR**", err);
                    }
                    var found = !!(index && Object.keys(index) && Object.keys(index)[0]);

                    if (found) {
                        var filePath = Object.keys(index)[0];
                        //var fileInfo = index[filePath];
                        var imageFormat = path.parse(filePath).ext.substr(1);
                        fs.readFile(filePath, function(err, buffer) {
                            if (err) {
                                res.sendStatus(500);
                                return log("**ERROR**", err);
                            }
                            lwip.open(buffer, imageFormat, function(err, image) {
                                if (err) {
                                    res.sendStatus(500);
                                    return log("**ERROR**", err);
                                }
                                image.scale(densityRatio * zoomLevel /** fileInfo.scaleWith*/ , function(err, image) {
                                    if (err) {
                                        res.sendStatus(500);
                                        return log("**ERROR**", err);
                                    }

                                    image.toBuffer(imageFormat, function(err, buffer) {
                                        if (err) {
                                            res.sendStatus(500);
                                            return log("**ERROR**", err);
                                        }
                                        res.type(imageFormat).
                                        set({
                                            "Pragma-directive": "no-cache",
                                            "Cache-directive": "no-cache",
                                            "Cache-control": "no-cache",
                                            "Pragma": "no-cache",
                                            "Expires": "0",
                                        }).
                                        send(buffer);
                                    });

                                });
                                // image.scale(wRatio, hRatio, inter, callback)
                            });
                        });
                    }
                    else {
                        res.sendStatus(404);
                    }
                });
        }
        else {
            ws.getImage(device, req.params.imageName,
                function indexResultAll(err, index) {
                    if (err) {
                        res.sendStatus(500);
                        return log("**ERROR**", err);
                    }
                    var result = {};
                    for (var p in index) {
                        var item = index[p];
                        item.path = p;
                        var parsedPath = path.parse(p);
                        var key = parsedPath.name.split("@")[0] + parsedPath.ext;
                        result[key] = item;
                    }
                    res.json(result);
                }, false);
        }
    });
};
